@@ -6,6 +6,7 @@ import os
 from lxml import etree
 
 from skrobot.urdf.modularize_urdf import find_root_link
+from skrobot.urdf.modularize_urdf import print_xacro_usage_instructions
 from skrobot.urdf.modularize_urdf import transform_urdf_to_macro
 
 
@@ -27,15 +28,8 @@ def main():
         output_path = base_name + "_modularized.xacro"
 
     etree.ElementTree(xacro_root).write(output_path, pretty_print=True, xml_declaration=True, encoding="utf-8")
-    print("Successfully converted to xacro: {}".format(output_path))
-    print("""To use the generated xacro macro in your xacro file, copy and paste the following:
 
-  <xacro:{}
-    prefix="[specify prefix]"
-    parent_link="[specify parent link]">
-    <origin xyz="0 0 0" rpy="0 0 0"/>
-  </xacro:{}>
-""".format(robot_name, robot_name))
+    print_xacro_usage_instructions(output_path, robot_name, args.no_prefix)
 
 
 if __name__ == "__main__":
