@@ -261,9 +261,11 @@ def sr_inverse(J, k=1.0, weight_vector=None):
 
     # umat = J W J^T + kI
     # ret = W J^T (J W J^T + kI)^(-1)
+    # Instead of computing inverse, solve Ax=b where A=umat, b=weight_J,
+    # x=ret.T
     weight_J = np.matmul(weight_matrix, J.T)
     umat = np.matmul(J, weight_J) + k * np.eye(r)
-    ret = np.matmul(weight_J, np.linalg.inv(umat))
+    ret = np.linalg.solve(umat, weight_J.T).T
     return ret
 
 
@@ -287,8 +289,10 @@ def sr_inverse_org(J, k=1.0):
         calculated SR-inverse
     """
     r, _ = J.shape
-    return np.matmul(J.T,
-                     np.linalg.inv(np.matmul(J, J.T) + k * np.eye(r)))
+    # We want J^T * (JJ^T + kI)^(-1)
+    # This equals (solve((JJ^T + kI), J))^T since (JJ^T + kI) is symmetric
+    umat = np.matmul(J, J.T) + k * np.eye(r)
+    return np.linalg.solve(umat, J).T
 
 
 def batch_sr_inverse(J_batch, k_values, weight_vector=None):
@@ -342,8 +346,10 @@ def batch_sr_inverse(J_batch, k_values, weight_vector=None):
     J_T = J_batch.transpose(0, 2, 1)
     weighted_J_T = np.matmul(W, J_T)
     umat = np.matmul(J_batch, weighted_J_T) + k_matrix
-    umat_inv = np.linalg.inv(umat)
-    sr_inverse = np.matmul(weighted_J_T, umat_inv)
+    # Instead of computing inverse, solve Ax=b for each matrix in batch
+    # A=umat[i], b=weighted_J_T[i], x=sr_inverse[i].T
+    sr_inverse = np.linalg.solve(
+        umat, weighted_J_T.transpose(0, 2, 1)).transpose(0, 2, 1)
     return sr_inverse
 
 
@@ -636,6 +642,10 @@ def rpy_matrix(az, ay, ax):
 def rpy_angle(matrix):
     """Decomposing a rotation matrix to yaw-pitch-roll.
 
+    .. deprecated::
+        This function is deprecated and confusing. Use matrix2ypr() or matrix2rpy() instead.
+        matrix2ypr() returns [yaw, pitch, roll] and matrix2rpy() returns [roll, pitch, yaw].
+
     Parameters
     ----------
     matrix : list or numpy.ndarray
@@ -659,6 +669,12 @@ def rpy_angle(matrix):
     (array([1.57079633, 1.04719755, 0.52359878]),
      array([ 4.71238898,  2.0943951 , -2.61799388]))
     """
+    warnings.warn(
+        "rpy_angle() is deprecated and confusing. Use matrix2ypr() for [yaw, pitch, roll] "
+        "or matrix2rpy() for [roll, pitch, yaw] instead.",
+        DeprecationWarning,
+        stacklevel=2
+    )
     if np.sqrt(matrix[1, 0] ** 2 + matrix[0, 0] ** 2) < _EPS:
         a = 0.0
     else:
@@ -679,6 +695,166 @@ def rpy_angle(matrix):
     return rpy, np.array([a, b, c])
 
 
+def matrix2ypr(matrix):
+    """Convert rotation matrix to yaw-pitch-roll angles.
+
+    This function extracts yaw, pitch, roll angles from a rotation matrix.
+
+    Parameters
+    ----------
+    matrix : numpy.ndarray
+        3x3 rotation matrix
+
+    Returns
+    -------
+    angles : numpy.ndarray
+        Array of [yaw, pitch, roll] angles in radians
+        - yaw: rotation around Z-axis
+        - pitch: rotation around Y-axis
+        - roll: rotation around X-axis
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from skrobot.coordinates.math import ypr2matrix, matrix2ypr
+    >>> yaw, pitch, roll = np.pi/6, np.pi/4, np.pi/3
+    >>> rot = ypr2matrix(yaw, pitch, roll)
+    >>> angles = matrix2ypr(rot)
+    >>> np.allclose(angles, [yaw, pitch, roll])
+    True
+    >>> print("Yaw: {:.3f}, Pitch: {:.3f}, Roll: {:.3f}".format(angles[0], angles[1], angles[2]))
+    Yaw: 0.524, Pitch: 0.785, Roll: 1.047
+    """
+    # Extract yaw, pitch, roll angles directly
+    if np.sqrt(matrix[1, 0] ** 2 + matrix[0, 0] ** 2) < _EPS:
+        yaw = 0.0
+    else:
+        yaw = np.arctan2(matrix[1, 0], matrix[0, 0])
+
+    sin_yaw = np.sin(yaw)
+    cos_yaw = np.cos(yaw)
+    pitch = np.arctan2(-matrix[2, 0], cos_yaw * matrix[0, 0] + sin_yaw * matrix[1, 0])
+    roll = np.arctan2(sin_yaw * matrix[0, 2] - cos_yaw * matrix[1, 2],
+                      -sin_yaw * matrix[0, 1] + cos_yaw * matrix[1, 1])
+
+    return np.array([yaw, pitch, roll])
+
+
+def matrix2rpy(matrix):
+    """Extract roll, pitch, yaw angles from rotation matrix.
+
+    This function returns angles in roll-pitch-yaw order (X-Y-Z rotation order).
+
+    Parameters
+    ----------
+    matrix : numpy.ndarray
+        3x3 rotation matrix
+
+    Returns
+    -------
+    angles : numpy.ndarray
+        Array of [roll, pitch, yaw] angles in radians
+        - roll: rotation around X-axis
+        - pitch: rotation around Y-axis
+        - yaw: rotation around Z-axis
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from skrobot.coordinates.math import rpy_matrix, matrix2rpy
+    >>> roll, pitch, yaw = np.pi/3, np.pi/4, np.pi/6
+    >>> rot = rpy_matrix(yaw, pitch, roll)  # Note: rpy_matrix takes yaw first
+    >>> angles = matrix2rpy(rot)
+    >>> np.allclose(angles, [roll, pitch, yaw])
+    True
+    >>> print("Roll: {:.3f}, Pitch: {:.3f}, Yaw: {:.3f}".format(angles[0], angles[1], angles[2]))
+    Roll: 1.047, Pitch: 0.785, Yaw: 0.524
+    """
+    ypr_angles = matrix2ypr(matrix)
+    # Convert from [yaw, pitch, roll] to [roll, pitch, yaw]
+    return np.array([ypr_angles[2], ypr_angles[1], ypr_angles[0]])
+
+
+def ypr2matrix(yaw, pitch, roll):
+    """Create rotation matrix from yaw, pitch, roll angles.
+
+    This is a clearly named wrapper around rpy_matrix that explicitly
+    indicates the parameter order. The rotation is applied in Z-Y-X order
+    (yaw around Z, then pitch around Y, then roll around X).
+
+    Parameters
+    ----------
+    yaw : float
+        Rotation around Z-axis in radians
+    pitch : float
+        Rotation around Y-axis in radians
+    roll : float
+        Rotation around X-axis in radians
+
+    Returns
+    -------
+    matrix : numpy.ndarray
+        3x3 rotation matrix
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from skrobot.coordinates.math import ypr2matrix, matrix2ypr
+    >>> yaw, pitch, roll = np.pi/6, np.pi/4, np.pi/3
+    >>> rot = ypr2matrix(yaw, pitch, roll)
+    >>> recovered = matrix2ypr(rot)
+    >>> np.allclose(recovered, [yaw, pitch, roll])
+    True
+
+    # Compare with rpy_matrix (which has same parameter order)
+    >>> from skrobot.coordinates.math import rpy_matrix
+    >>> rot_old = rpy_matrix(yaw, pitch, roll)
+    >>> np.allclose(rot, rot_old)
+    True
+    """
+    return rpy_matrix(yaw, pitch, roll)
+
+
+def rpy2matrix(roll, pitch, yaw):
+    """Create rotation matrix from roll, pitch, yaw angles.
+
+    This function takes parameters in roll-pitch-yaw order (X-Y-Z rotation order),
+    which is more intuitive for many applications. The rotation is applied as:
+    roll around X, then pitch around Y, then yaw around Z.
+
+    Parameters
+    ----------
+    roll : float
+        Rotation around X-axis in radians
+    pitch : float
+        Rotation around Y-axis in radians
+    yaw : float
+        Rotation around Z-axis in radians
+
+    Returns
+    -------
+    matrix : numpy.ndarray
+        3x3 rotation matrix
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from skrobot.coordinates.math import rpy2matrix, matrix2rpy
+    >>> roll, pitch, yaw = np.pi/3, np.pi/4, np.pi/6
+    >>> rot = rpy2matrix(roll, pitch, yaw)
+    >>> recovered = matrix2rpy(rot)
+    >>> np.allclose(recovered, [roll, pitch, yaw])
+    True
+
+    # Verify rotation order: X then Y then Z
+    >>> # Pure rotations should work correctly
+    >>> rot_x = rpy2matrix(np.pi/2, 0, 0)
+    >>> rot_y = rpy2matrix(0, np.pi/2, 0)
+    >>> rot_z = rpy2matrix(0, 0, np.pi/2)
+    """
+    return rpy_matrix(yaw, pitch, roll)
+
+
 def normalize_vector(v, ord=2):
     """Return normalized vector
 
@@ -703,9 +879,16 @@ def normalize_vector(v, ord=2):
     array([0., 0., 0.])
     """
     v = np.array(v, dtype=np.float64)
-    norm = np.linalg.norm(v, ord=ord)
-    if norm == 0:
-        return v
+    if ord == 2:
+        # Optimized L2 norm calculation
+        norm_squared = np.dot(v, v)
+        if norm_squared == 0:
+            return v
+        norm = np.sqrt(norm_squared)
+    else:
+        norm = np.linalg.norm(v, ord=ord)
+        if norm == 0:
+            return v
     return v / norm
 
 
@@ -986,7 +1169,12 @@ def cross_product(a, b):
     cross_prod : numpy.ndarray
         calculated cross product
     """
-    return np.dot(outer_product_matrix(a), b)
+    # Direct implementation instead of matrix multiplication for better performance
+    return np.array([
+        a[1] * b[2] - a[2] * b[1],
+        a[2] * b[0] - a[0] * b[2],
+        a[0] * b[1] - a[1] * b[0]
+    ])
 
 
 def quaternion2rpy(q):
@@ -1926,6 +2114,60 @@ def make_matrix(r, c):
     return np.zeros((r, c), 'f')
 
 
+def invert_yaw_pitch_roll(yaw=0.0, pitch=0.0, roll=0.0):
+    """Calculate the inverse of yaw-pitch-roll Euler angles.
+
+    This function computes the inverse transformation of a rotation specified
+    by yaw-pitch-roll angles. When used with rpy_matrix(), it produces
+    results compatible with the matrix inversion.
+
+    Parameters
+    ----------
+    yaw : float
+        Rotation angle around Z-axis in radians
+    pitch : float
+        Rotation angle around Y-axis in radians
+    roll : float
+        Rotation angle around X-axis in radians
+
+    Returns
+    -------
+    yaw_inv : float
+        Inverse yaw angle in radians
+    pitch_inv : float
+        Inverse pitch angle in radians
+    roll_inv : float
+        Inverse roll angle in radians
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from skrobot.coordinates.math import invert_yaw_pitch_roll
+    >>> yaw, pitch, roll = np.pi/4, np.pi/6, np.pi/3
+    >>> yaw_inv, pitch_inv, roll_inv = invert_yaw_pitch_roll(yaw, pitch, roll)
+    """
+    from scipy.spatial.transform import Rotation
+    # Create rotation matrix using rpy_matrix convention for consistency
+    R_original = rpy_matrix(yaw, pitch, roll)
+    # Convert to SciPy Rotation object and invert
+    R_inv = Rotation.from_matrix(R_original).inv()
+    # Get inverse rotation matrix and extract rpy angles
+    R_inv_matrix = R_inv.as_matrix()
+
+    # Extract rpy angles from the inverse matrix (same logic as rpy_angle)
+    if np.sqrt(R_inv_matrix[1, 0] ** 2 + R_inv_matrix[0, 0] ** 2) < _EPS:
+        a = 0.0
+    else:
+        a = np.arctan2(R_inv_matrix[1, 0], R_inv_matrix[0, 0])
+    sa = np.sin(a)
+    ca = np.cos(a)
+    b = np.arctan2(-R_inv_matrix[2, 0], ca * R_inv_matrix[0, 0] + sa * R_inv_matrix[1, 0])
+    c = np.arctan2(sa * R_inv_matrix[0, 2] - ca * R_inv_matrix[1, 2],
+                    -sa * R_inv_matrix[0, 1] + ca * R_inv_matrix[1, 1])
+
+    return a, b, c
+
+
 inverse_rodrigues = rotation_angle
 quat_from_rotation_matrix = matrix2quaternion
 quat_from_rpy = rpy2quaternion
